==> linuxconf-tools-1.1.src.tar.gz: linuxconf-tools-1.1/projetx/bfusion.cc <==
#include <string.h>
#include <misc.h>
#include <dialog.h>
#include <tool.h>
#include "projet.h"
#include "projetx.m"
#include "fusion.h"
#include "prjlist.h"

/* #Spcification: fusion / introduction
	La fusion permet de combiner diffrentes rvisions et
	produire une nouvelle. Il y a deux cas:

	La fusion simple et le cauchemar.

	La fusion simple est l'opration qui permet de combiner
	les modifications de rvisions qui possdent la mme rvision
	rfrence.

	exemple:
		Rvision rfrence 1.0
		Le programmeur A commande cette rvision et la modifie. Il
		livre 1.0.1.1.
		Le programmeur B fait de mme et livre 1.0.2.1.

			  +-- 1.0.1.1 ---+
		1.0 --|              |-- 1.1
			  +-- 1.0.2.1 ---+

		On dit que cette fusion est simple parce que les changements
		qui ont t fait peuvent facilement tre dtermin en
		comparant avec la rvision 1.0.

	Le cauchemar se produit lorsqu'on doit travailler toute une
	nuit pour faire une fusion. Paradoxalement, on ne dort pas
	durant le cauchemar.

	La fusion complexe (dite cauchemar) se produit lorsqu'on
	combine des rvisions qui n'ont pas la mme rvision rfrence.
	Ou encore ont une rvision rfrences commune tres lointaine
	(ultimement 0.0).

	Voici un exemple de cauchemar que Picasso lui-mme ne saurait
	peindre, lui qui pourtant voyait tout de travers.

		  +- 0.0.1.1 -+			+-- 0.2.1.1 -+...- 0.2.1.9 -+
		  |           |-- 0.2 --|			 |			    |
		  +- 0.0.2.1 -+			+-- 0.2.2.1 -+			    |- 0.12
		  |									 |			    |
	0.0 --|									 |- 0.11 -------+
		  +- 0.0.3.1 -+						 |
		  +- 0.0.4.1 -+-- 0.1 -- ... 0.10 ---+
		  +- 0.0.5.1 -+


	Dans le dessin suivant, on veut fusionner 0.11 avec 0.2.1.9.
	Ce qui nous interessent, ce sont les changement qui ont t
	fait depuis 0.2.1.1. Ceci est particulirement complexe
	parce que la version 0.2.1.9 est trs diffrentes de la
	version 0.11 et aussi 0.10.

	Pour s'en sortir, on choisit la rvision de base: 0.11.
	Ensuite on choisit une rvision rfrence: 0.2.1.1.
	On choisit la rvision  fusionner: 0.2.1.9. On associe
	cette dernire  0.2.1.1 (sa rfrence). Le programme
	tablira la liste de tous les fichiers qui ont chang entre
	0.2.1.1 et 0.2.1.9 et permettra  l'usager de comparer les
	version 0.11 0.2.1.1 et 0.2.1.9.
*/
#

/*
	Met  jour le fichier version.dat pour "rserver" la nouvelle rvision.
*/
static void near bfusion_setintgrev(
	USERINFO *intg,
	VERSION_DAT *version,
	VERSION_PAIRE tbpaire[],
	int nbvar)
{
	/* #Specification: fusion / extraction
		Lorsque l'on extrait l'information pour faire
		une intgration dans l'espace commun, la version destination est
		cre immdiatement avec des commentaires
		permettant de connaitre son origine.

		Le fichier version.dat est immdiatement corrig.
	*/
	char path[MAXSIZ_PATH];
	intg->makrefpath("",path);
	path_make (path,"version.dat",path);
	const REVISION *revdst = intg->getrevdst();
	VERSION_ONE *one = version->get (revdst);
	// Si one existe, c'est qu'on fait un intgration dans l'espace usager
	if (one == NULL) one = version->add (intg->getversion(),NULL,revdst);
	char revstr[MAXSIZ_REVISION];
	intg->getrevsrc()->format (revstr);
	one->addinfo ("Intgration de %s",revstr);
	for (int i=0; i<nbvar; i++){
		VERSION_ONE *rev = tbpaire[i].var;
		rev->rev.format (revstr);
		one->addinfo ("             + %s",revstr);
	}
	version->save(intg);
}

/*
	Balaye les sous-rpertoires et prpare les integre.dat
*/
static int near bfusion_setintg (
	USERINFO *user,
	VERSION_DAT &version,
	VERSION_PAIRE tbpaire[],
	int nbvar,
	const char *tbprj[],
	int nbprj)
{
	int ret = -1;
	WINDOW_LOG log ("fusion.log",1);
	log.show();
	if (projet_chkusrenv(user) != -1){
		ret = projet_get   (user,tbprj,nbprj,0,1,0,&log);
		if (ret != -1){
			/*
				Il faut s'assurer que les sources de toutes les
				rvisions dans tbvar sont disponibles dans kit/ombre.
				On fait simplement un extraction dans \kit\build.
			*/
			/*
				On doit aussi produire pour chaque rpertoires
				de tous les projets slectionns, un rsum des
				diffrences entre la version de base et la version
				intgr. Ce fichier permettra de rsoudre les conflits,
				de visualiser les fichiers qui n'existe plus et de voir
				ceux qui se sont ajouts.

				L'option edition-fusion utilisera ce rsum pour
				piloter le programmeur responsable de l'intgration.
			*/
			for (int i=0; i<nbvar && ret != -1; i++){
				USERBUILD build (tbpaire[i].var->getversion()
					,&tbpaire[i].var->rev);
				USERBUILD ref (tbpaire[i].ref->getversion()
					,&tbpaire[i].ref->rev);
				ret = bfusion_extractrev (&build,&ref,user
					,tbprj,nbprj,&log);
			}
			if (ret != -1){
				bfusion_setintgrev (user,&version,tbpaire,nbvar);
			}
		}
	}
	return ret;
}
// Extraction de plusieurs version intermdiaire pour intgration
// dans l'environnement de recompilation.
class PRJLISTE_INTEGRE: public PRJLISTE{
	VERSION_PAIRE *tbpaire;
	int nbpaire;
	VERSION_DAT &version;
	/*~PROTOBEG~ PRJLISTE_INTEGRE */
public:
	PRJLISTE_INTEGRE (VERSION_PAIRE _tbpaire[],
		 int _nbpaire,
		 VERSION_DAT&_version);
	int exec (USERINFO *user,
		 int opr,
		 const char *tbprj[],
		 int nbprj);
	const char **getbutton (void);
	int getlist (USERINFO *user,
		 PRJ_INFO tbprj[],
		 int);
	/*~PROTOEND~ PRJLISTE_INTEGRE */
};

PUBLIC PRJLISTE_INTEGRE::PRJLISTE_INTEGRE(
	VERSION_PAIRE _tbpaire[],
	int _nbpaire,
	VERSION_DAT &_version)
	: tbpaire(_tbpaire), nbpaire(_nbpaire), version(_version)
{
}
	
PUBLIC int PRJLISTE_INTEGRE::getlist(
	USERINFO *user,
	PRJ_INFO tbprj[],	// Contiendra les noms de projets a intgrer
	int )		// maxtb
{
	int nbprj = 0;
	VERSION_PAIRE *ptp = tbpaire;
	/* Spcification: intgration / liste de projet / attribut
		La liste des projets slectionnable est l'union de tous les
		projet membre des rvisions  fusionner et non pas seulement
		la liste des projets de la rvision rfrence.

		Pour aider  la slection des projets a fusionner, spcialement
		pour les fusions en espace usager, on prsente vis  vis chaque
		projet un petit message (attribut). Ce message permet de visualiser
		rapidement les projets "actifs".

		Voici la liste des messages:

			+	: Indique qu'une rvision contient une rvision modifie
				  de ce projet.
			*	: Indique que plusieurs rvisions contiennent une rvision
				  modifies de ce projets
			rien: Aucune rvision n'a modifi ce projet.
	*/
	for (int i=0; i<nbpaire; i++, ptp++){
		MAKEFILE mkfvar (NULL,0,ptp->var->getrev(),"",user);
		MAKEFILE mkfref (NULL,0,ptp->ref->getrev(),"",user);
		mkfvar.setiter();
		MAKEFILE_FILE *var_file;
		while ((var_file=mkfvar.iter("*/makefile.dat",0))!=NULL){
			const char *nom = var_file->getnom();
			PRJ_INFO *ptprj = tbprj;
			int j;
			for (j=0; j<nbprj; j++,ptprj++){
				if (strcmp(ptprj->nom,nom)==0) break;
			}
			if (j==nbprj){
				ptprj->nom = strdup_err (nom);
				nbprj++;
			}
			MAKEFILE_FILE *ref_file = mkfref.locate(nom);
			if (ref_file == NULL
				|| ref_file->getrev()->cmp(var_file->getrev())!=0){
				char *ptatr = &ptprj->atr;
				if (*ptatr == ' '){
					*ptatr = '+';
				}else if (*ptatr == '+'){
					*ptatr = '*';
				}
			}
		}
	}
	// Enlve le /makefile.dat
	PRJ_INFO *ptprj = tbprj;
	for (int i=0; i<nbprj; i++, ptprj++){
		path_splitlex (ptprj->nom,ptprj->nom,NULL);
	}
	return nbprj;
}
PUBLIC int PRJLISTE_INTEGRE::exec (
	USERINFO *user,
	int opr,			// Opration command.
	const char *tbprj[],	// Projets slectionns
	int nbprj)
{
	int ret = -1;
	if (nbprj == 0){
		xconf_error (MSG_U(E_AUMOINS1
			,"C'est pas en ne choisissant rien\n"
			 "que vous allez amliorer votre productivit."));
	}else if (opr == 0){
		ret = bfusion_setintg (user,version
				,tbpaire,nbpaire,tbprj,nbprj);
	}
	return ret;
}

PUBLIC const char **PRJLISTE_INTEGRE::getbutton()
{
	static const char *tb[]={MSG_U(B_INTEGRE,"Intgre"),NULL};
	return tb;
}

/*
	Installation d'une combinaison de rvision pour intgration.

	Cette extraction servira  faire la fusion des modifications et
	faire la recompilation du projet.

	Retourne -1 si abandon ou erreur.
*/
int bfusion_setup (
	USERINFO *user,
	int fusion_usager)		// Les fichiers integre.dat
							// sont accumuls dans l'environnement dfinie
							// par user, ou dans une nouvelle rvision
							// de /kit/build
{
	int ret = -1;
	PRJLOCK racine(user,"_racine");
	if (racine.isok()){
		VERSION_PAIRE tbpaire[50];
		VERSION_DAT version(user);
		#ifdef MISSING
			int nbvar = bfusion_compose (user,tbpaire,version);
		#else
			int nbvar = 0;
		#endif
		if (nbvar > 0){
			PRJLISTE_INTEGRE g(tbpaire,nbvar,version);
			if (fusion_usager){
				projet_select (user,g);
			}else{
				REVISION     revnext;		// La rvision qu'on veut produire.
				VERSION_ONE *cur = version.get(user->getrevdst());
				if (cur != NULL && cur->isinteg()){
					/* #Spcification: fusion / livraison tardive
						Si on prpare une fusion dans une fusion non
						livre, les fichiers integre.dat seront ajouts
						 cette rvision. Cela permet de dbuter une
						intgration et d'ajouter des livraisons  celle-ci
						en cours de route.

						Cela rgle le problme suivant: L'intgrateur a dj
						commencer et un programmeur lui demande "J'ai
						finalement rgler un problme important, est-ce trop
						tard ?". La rponse pourra tre "non" maintenant.
					*/
					revnext = *user->getrevdst();
				}else{
					/*
						Trouve le plus grand numro de rvision
						pour la version de base. Etablie le numro
						de rvision pour l'intgration  partir de
						cette rvision.
					*/
					VERSION_ONE *nextintg = version.select (user->getversion()
						,NULL);
					revnext = nextintg->rev;
					revnext.next(0,REVISION_INTEGRE);
				}


				/* #Spcification: fusion / tapes / installation
					Aprs avoir slectionner les diffrentes rvisions
					requises pour la fusion, il faut installer une copie
					de la rvision de base dans l'environnement d'intgration
					/kit/build. On installe immdiatement cette rvision sous
					le numro de la rvision qui sera produite par l'intgration.


					exemple:

						1.0 + ( 1.0.1.1 , 1.0.2.1, 1.0.3.1 ) -> 1.1

						La rvision 1.0 va tre install dans /kit/build/_1_1
				*/
				USERINTG intg (user->getversion(),user->getrevsrc(),&revnext);
				projet_select (&intg,g);
			}
		}
	}
	return ret;
}

==> linuxconf-tools-1.7.src.tar.gz: linuxconf-tools-1.7/projetx/bfusion.cc <==
#include <string.h>
#include <misc.h>
#include <dialog.h>
#include <tool.h>
#include "projet.h"
#include "projetx.m"
#include "fusion.h"
#include "prjlist.h"

/* #Spcification: fusion / introduction
	La fusion permet de combiner diffrentes rvisions et
	produire une nouvelle. Il y a deux cas:

	La fusion simple et le cauchemar.

	La fusion simple est l'opration qui permet de combiner
	les modifications de rvisions qui possdent la mme rvision
	rfrence.

	exemple:
		Rvision rfrence 1.0
		Le programmeur A commande cette rvision et la modifie. Il
		livre 1.0.1.1.
		Le programmeur B fait de mme et livre 1.0.2.1.

			  +-- 1.0.1.1 ---+
		1.0 --|              |-- 1.1
			  +-- 1.0.2.1 ---+

		On dit que cette fusion est simple parce que les changements
		qui ont t fait peuvent facilement tre dtermin en
		comparant avec la rvision 1.0.

	Le cauchemar se produit lorsqu'on doit travailler toute une
	nuit pour faire une fusion. Paradoxalement, on ne dort pas
	durant le cauchemar.

	La fusion complexe (dite cauchemar) se produit lorsqu'on
	combine des rvisions qui n'ont pas la mme rvision rfrence.
	Ou encore ont une rvision rfrences commune tres lointaine
	(ultimement 0.0).

	Voici un exemple de cauchemar que Picasso lui-mme ne saurait
	peindre, lui qui pourtant voyait tout de travers.

		  +- 0.0.1.1 -+			+-- 0.2.1.1 -+...- 0.2.1.9 -+
		  |           |-- 0.2 --|			 |			    |
		  +- 0.0.2.1 -+			+-- 0.2.2.1 -+			    |- 0.12
		  |									 |			    |
	0.0 --|									 |- 0.11 -------+
		  +- 0.0.3.1 -+						 |
		  +- 0.0.4.1 -+-- 0.1 -- ... 0.10 ---+
		  +- 0.0.5.1 -+


	Dans le dessin suivant, on veut fusionner 0.11 avec 0.2.1.9.
	Ce qui nous interessent, ce sont les changement qui ont t
	fait depuis 0.2.1.1. Ceci est particulirement complexe
	parce que la version 0.2.1.9 est trs diffrentes de la
	version 0.11 et aussi 0.10.

	Pour s'en sortir, on choisit la rvision de base: 0.11.
	Ensuite on choisit une rvision rfrence: 0.2.1.1.
	On choisit la rvision  fusionner: 0.2.1.9. On associe
	cette dernire  0.2.1.1 (sa rfrence). Le programme
	tablira la liste de tous les fichiers qui ont chang entre
	0.2.1.1 et 0.2.1.9 et permettra  l'usager de comparer les
	version 0.11 0.2.1.1 et 0.2.1.9.
*/
#

/*
	Met  jour le fichier version.dat pour "rserver" la nouvelle rvision.
*/
static void near bfusion_setintgrev(
	USERINFO *intg,
	VERSION_DAT *version,
	VERSION_PAIRE tbpaire[],
	int nbvar)
{
	/* #Specification: fusion / extraction
		Lorsque l'on extrait l'information pour faire
		une intgration dans l'espace commun, la version destination est
		cre immdiatement avec des commentaires
		permettant de connaitre son origine.

		Le fichier version.dat est immdiatement corrig.
	*/
	char path[MAXSIZ_PATH];
	intg->makrefpath("",path);
	path_make (path,"version.dat",path);
	const REVISION *revdst = intg->getrevdst();
	VERSION_ONE *one = version->get (revdst);
	// Si one existe, c'est qu'on fait un intgration dans l'espace usager
	if (one == NULL) one = version->add (intg->getversion(),NULL,revdst);
	char revstr[MAXSIZ_REVISION];
	intg->getrevsrc()->format (revstr);
	one->addinfo ("Intgration de %s",revstr);
	for (int i=0; i<nbvar; i++){
		VERSION_ONE *rev = tbpaire[i].var;
		rev->rev.format (revstr);
		one->addinfo ("             + %s",revstr);
	}
	version->save(intg);
}

/*
	Balaye les sous-rpertoires et prpare les integre.dat
*/
static int near bfusion_setintg (
	USERINFO *user,
	VERSION_DAT &version,
	VERSION_PAIRE tbpaire[],
	int nbvar,
	const char *tbprj[],
	int nbprj)
{
	int ret = -1;
	WINDOW_LOG log ("fusion.log",1);
	log.show();
	if (projet_chkusrenv(user) != -1){
		ret = projet_get   (user,tbprj,nbprj,false,true,false,&log);
		if (ret != -1){
			/*
				Il faut s'assurer que les sources de toutes les
				rvisions dans tbvar sont disponibles dans kit/ombre.
				On fait simplement un extraction dans \kit\build.
			*/
			/*
				On doit aussi produire pour chaque rpertoires
				de tous les projets slectionns, un rsum des
				diffrences entre la version de base et la version
				intgr. Ce fichier permettra de rsoudre les conflits,
				de visualiser les fichiers qui n'existe plus et de voir
				ceux qui se sont ajouts.

				L'option edition-fusion utilisera ce rsum pour
				piloter le programmeur responsable de l'intgration.
			*/
			for (int i=0; i<nbvar && ret != -1; i++){
				USERBUILD build (tbpaire[i].var->getversion()
					,&tbpaire[i].var->rev);
				USERBUILD ref (tbpaire[i].ref->getversion()
					,&tbpaire[i].ref->rev);
				ret = bfusion_extractrev (&build,&ref,user
					,tbprj,nbprj,&log);
			}
			if (ret != -1){
				bfusion_setintgrev (user,&version,tbpaire,nbvar);
			}
		}
	}
	return ret;
}
// Extraction de plusieurs version intermdiaire pour intgration
// dans l'environnement de recompilation.
class PRJLISTE_INTEGRE: public PRJLISTE{
	VERSION_PAIRE *tbpaire;
	int nbpaire;
	VERSION_DAT &version;
	/*~PROTOBEG~ PRJLISTE_INTEGRE */
public:
	PRJLISTE_INTEGRE (VERSION_PAIRE _tbpaire[],
		 int _nbpaire,
		 VERSION_DAT&_version);
	int exec (USERINFO *user,
		 int opr,
		 const char *tbprj[],
		 int nbprj);
	const char **getbutton (void);
	int getlist (USERINFO *user,
		 PRJ_INFO tbprj[],
		 int);
	/*~PROTOEND~ PRJLISTE_INTEGRE */
};

PUBLIC PRJLISTE_INTEGRE::PRJLISTE_INTEGRE(
	VERSION_PAIRE _tbpaire[],
	int _nbpaire,
	VERSION_DAT &_version)
	: tbpaire(_tbpaire), nbpaire(_nbpaire), version(_version)
{
}
	
PUBLIC int PRJLISTE_INTEGRE::getlist(
	USERINFO *user,
	PRJ_INFO tbprj[],	// Contiendra les noms de projets a intgrer
	int )		// maxtb
{
	int nbprj = 0;
	VERSION_PAIRE *ptp = tbpaire;
	/* Spcification: intgration / liste de projet / attribut
		La liste des projets slectionnable est l'union de tous les
		projet membre des rvisions  fusionner et non pas seulement
		la liste des projets de la rvision rfrence.

		Pour aider  la slection des projets a fusionner, spcialement
		pour les fusions en espace usager, on prsente vis  vis chaque
		projet un petit message (attribut). Ce message permet de visualiser
		rapidement les projets "actifs".

		Voici la liste des messages:

			+	: Indique qu'une rvision contient une rvision modifie
				  de ce projet.
			*	: Indique que plusieurs rvisions contiennent une rvision
				  modifies de ce projets
			rien: Aucune rvision n'a modifi ce projet.
	*/
	for (int i=0; i<nbpaire; i++, ptp++){
		MAKEFILE mkfvar (NULL,false,ptp->var->getrev(),"",user);
		MAKEFILE mkfref (NULL,false,ptp->ref->getrev(),"",user);
		mkfvar.setiter();
		MAKEFILE_FILE *var_file;
		while ((var_file=mkfvar.iter("*/makefile.dat",0))!=NULL){
			const char *nom = var_file->getnom();
			PRJ_INFO *ptprj = tbprj;
			int j;
			for (j=0; j<nbprj; j++,ptprj++){
				if (strcmp(ptprj->nom,nom)==0) break;
			}
			if (j==nbprj){
				ptprj->nom = strdup_err (nom);
				nbprj++;
			}
			MAKEFILE_FILE *ref_file = mkfref.locate(nom);
			if (ref_file == NULL
				|| ref_file->getrev()->cmp(var_file->getrev())!=0){
				char *ptatr = &ptprj->atr;
				if (*ptatr == ' '){
					*ptatr = '+';
				}else if (*ptatr == '+'){
					*ptatr = '*';
				}
			}
		}
	}
	// Enlve le /makefile.dat
	PRJ_INFO *ptprj = tbprj;
	for (int i=0; i<nbprj; i++, ptprj++){
		path_splitlex (ptprj->nom,ptprj->nom,NULL);
	}
	return nbprj;
}
PUBLIC int PRJLISTE_INTEGRE::exec (
	USERINFO *user,
	int opr,			// Opration command.
	const char *tbprj[],	// Projets slectionns
	int nbprj)
{
	int ret = -1;
	if (nbprj == 0){
		xconf_error (MSG_U(E_AUMOINS1
			,"C'est pas en ne choisissant rien\n"
			 "que vous allez amliorer votre productivit."));
	}else if (opr == 0){
		ret = bfusion_setintg (user,version
				,tbpaire,nbpaire,tbprj,nbprj);
	}
	return ret;
}

PUBLIC const char **PRJLISTE_INTEGRE::getbutton()
{
	static const char *tb[]={MSG_U(B_INTEGRE,"Intgre"),NULL};
	return tb;
}

/*
	Installation d'une combinaison de rvision pour intgration.

	Cette extraction servira  faire la fusion des modifications et
	faire la recompilation du projet.

	Retourne -1 si abandon ou erreur.
*/
int bfusion_setup (
	USERINFO *user,
	int fusion_usager)		// Les fichiers integre.dat
							// sont accumuls dans l'environnement dfinie
							// par user, ou dans une nouvelle rvision
							// de /kit/build
{
	int ret = -1;
	PRJLOCK racine(user,"_racine");
	if (racine.isok()){
		VERSION_PAIRE tbpaire[50];
		VERSION_DAT version(user);
		#ifdef MISSING
			int nbvar = bfusion_compose (user,tbpaire,version);
		#else
			int nbvar = 0;
		#endif
		if (nbvar > 0){
			PRJLISTE_INTEGRE g(tbpaire,nbvar,version);
			if (fusion_usager){
				projet_select (user,g);
			}else{
				REVISION     revnext;		// La rvision qu'on veut produire.
				VERSION_ONE *cur = version.get(user->getrevdst());
				if (cur != NULL && cur->isinteg()){
					/* #Spcification: fusion / livraison tardive
						Si on prpare une fusion dans une fusion non
						livre, les fichiers integre.dat seront ajouts
						 cette rvision. Cela permet de dbuter une
						intgration et d'ajouter des livraisons  celle-ci
						en cours de route.

						Cela rgle le problme suivant: L'intgrateur a dj
						commencer et un programmeur lui demande "J'ai
						finalement rgler un problme important, est-ce trop
						tard ?". La rponse pourra tre "non" maintenant.
					*/
					revnext = *user->getrevdst();
				}else{
					/*
						Trouve le plus grand numro de rvision
						pour la version de base. Etablie le numro
						de rvision pour l'intgration  partir de
						cette rvision.
					*/
					VERSION_ONE *nextintg = version.select (user->getversion()
						,NULL);
					revnext = nextintg->rev;
					revnext.next(0,REVISION_INTEGRE);
				}


				/* #Spcification: fusion / tapes / installation
					Aprs avoir slectionner les diffrentes rvisions
					requises pour la fusion, il faut installer une copie
					de la rvision de base dans l'environnement d'intgration
					/kit/build. On installe immdiatement cette rvision sous
					le numro de la rvision qui sera produite par l'intgration.


					exemple:

						1.0 + ( 1.0.1.1 , 1.0.2.1, 1.0.3.1 ) -> 1.1

						La rvision 1.0 va tre install dans /kit/build/_1_1
				*/
				USERINTG intg (user->getversion(),user->getrevsrc(),&revnext);
				projet_select (&intg,g);
			}
		}
	}
	return ret;
}
